@@ -19,26 +19,27 @@ class PyflakesDiagnosticReport(object):
     def unexpectedError(self, filename, msg):  # pragma: no cover
         pass
 
-    def syntaxError(self, filename, msg, lineno, offset, text):
-        range = {
+    def syntaxError(self, _filename, msg, lineno, offset, text):
+        err_range = {
             'start': {'line': lineno - 1, 'character': offset},
             'end': {'line': lineno - 1, 'character': offset + len(text)},
         }
         self.diagnostics.append({
             'source': 'pyflakes',
-            'range': range,
-            'message': msg
+            'range': err_range,
+            'message': msg,
+            'severity': lsp.DiagnosticSeverity.Error,
         })
 
     def flake(self, message):
         """ Get message like <filename>:<lineno>: <msg> """
-        range = {
+        err_range = {
             'start': {'line': message.lineno - 1, 'character': message.col},
             'end': {'line': message.lineno - 1, 'character': len(self.lines[message.lineno - 1])},
         }
         self.diagnostics.append({
             'source': 'pyflakes',
-            'range': range,
+            'range': err_range,
             'message': message.message % message.message_args,
             'severity': lsp.DiagnosticSeverity.Warning
         })
