@@ -1,6 +1,5 @@
 # Copyright 2017 Palantir Technologies, Inc.
 import logging
-from multiprocessing import dummy as multiprocessing
 from . import lsp, _utils
 from .config import config
 from .language_server import LanguageServer
@@ -8,7 +7,6 @@ from .workspace import Workspace
 
 log = logging.getLogger(__name__)
 
-PLUGGY_RACE_POOL_SIZE = 5
 LINT_DEBOUNCE_S = 0.5  # 500 ms
 
 
@@ -18,17 +16,32 @@ class PythonLanguageServer(LanguageServer):
     workspace = None
     config = None
 
-    _pool = multiprocessing.Pool(PLUGGY_RACE_POOL_SIZE)
+    # Set of method dispatchers to query
+    _dispatchers = []
+
+    def __getitem__(self, item):
+        """Override the method dispatcher to farm out any unknown messages to our plugins."""
+        try:
+            return super(PythonLanguageServer, self).__getitem__(item)
+        except KeyError:
+            log.debug("Checking dispatchers for %s: %s", item, self._dispatchers)
+            for dispatcher in self._dispatchers:
+                try:
+                    return dispatcher.__getitem__(item)
+                except KeyError:
+                    pass
+        raise KeyError("Unknown item %s" % item)
 
     def _hook_caller(self, hook_name):
         return self.config.plugin_manager.subset_hook_caller(hook_name, self.config.disabled_plugins)
 
     def _hook(self, hook_name, doc_uri=None, **kwargs):
         doc = self.workspace.get_document(doc_uri) if doc_uri else None
-        return self._hook_caller(hook_name)(config=self.config, workspace=self.workspace, document=doc, **kwargs)
+        hook = self.config.plugin_manager.subset_hook_caller(hook_name, self.config.disabled_plugins)
+        return hook(config=self.config, workspace=self.workspace, document=doc, **kwargs)
 
     def capabilities(self):
-        return {
+        server_capabilities = {
             'codeActionProvider': True,
             'codeLensProvider': {
                 'resolveProvider': False,  # We may need to make this configurable
@@ -50,12 +63,16 @@ class PythonLanguageServer(LanguageServer):
             'signatureHelpProvider': {
                 'triggerCharacters': ['(', ',']
             },
-            'textDocumentSync': lsp.TextDocumentSyncKind.INCREMENTAL
+            'textDocumentSync': lsp.TextDocumentSyncKind.INCREMENTAL,
+            'experimental': merge(self._hook('pyls_experimental_capabilities'))
         }
+        log.info("Server capabilities: %s", server_capabilities)
+        return server_capabilities
 
     def initialize(self, root_uri, init_opts, _process_id):
         self.workspace = Workspace(root_uri, lang_server=self)
         self.config = config.Config(root_uri, init_opts)
+        self._dispatchers = self._hook('pyls_dispatchers')
         self._hook('pyls_initialize')
 
     def code_actions(self, doc_uri, range, context):
@@ -65,14 +82,10 @@ class PythonLanguageServer(LanguageServer):
         return flatten(self._hook('pyls_code_lens', doc_uri))
 
     def completions(self, doc_uri, position):
-        completions = _utils.race_hooks(
-            self._hook_caller('pyls_completions'), self._pool,
-            document=self.workspace.get_document(doc_uri) if doc_uri else None,
-            position=position
-        )
+        completions = self._hook('pyls_completions', doc_uri, position=position)
         return {
             'isIncomplete': False,
-            'items': completions or []
+            'items': flatten(completions)
         }
 
     def definitions(self, doc_uri, position):
@@ -183,3 +196,7 @@ class PythonLanguageServer(LanguageServer):
 
 def flatten(list_of_lists):
     return [item for lst in list_of_lists for item in lst]
+
+
+def merge(list_of_dicts):
+    return {k: v for dictionary in list_of_dicts for k, v in dictionary.items()}
