@@ -64,7 +64,5 @@ def test_jedi_completion_ordering():
 
     items = {c['label']: c['sortText'] for c in completions}
 
-    # Assert that builtins come after our own functions even if alphabetically they're before
-    assert items['hello()'] < items['dict']
     # And that 'hidden' functions come after unhidden ones
     assert items['hello()'] < items['_a_hello()']
